/*
 * Copyright (C) 2002 sauron@kernsh.org
 * Copyright (C) 2002 zorgon@kernsh.org
 *
 * THIS LICENCE IS ISSUED FROM NMAP LICENCE AND MODIFIED
 *
 * The program kernsh is (C) 2002 sauron/zorgon. This program is
 * free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free
 * Software Foundation; Version 2. This guarantees your right to use,
 * modify, and redistribute this software under certain conditions.
 *
 * Source is provided to this software because we believe users have a
 * right to know exactly what a program is going to do before they run
 * it.
 *
 * Source code also allows you to fix bugs, and add new features.
 * You are highly encouraged to send your changes to dev@kernsh.org for
 * possible incorporation into the main distribution. By sending these
 * changes to sauron@kernsh.org or zorgon@kernsh.org, it is assumed that
 * you are offering them the unlimited, non-exclusive right to reuse,
 * modify, and relicense the code. This is important because the
 * inability to relicense code has caused devastating problems for other
 * Free Software projects (such as KDE and NASM). Kernsh will always be
 * available Open Source. If you wish to specify special license
 * conditions of your contributions, just say so when you send them.
 *
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details (
 * http://www.gnu.org/copyleft/gpl.html ).
 *
 */

#define MODULE
#define __KERNEL__
#include <linux/module.h>
#include <linux/kernel.h>
#include <sys/syscall.h>

#define SYS_HOOKED 250
#define MAX_PARAM 10

#define T_INT 'i'
#define T_CHAR 'c'
#define T_STRING 's'
#define T_POINTER 'p'
#define T_SHORT 'u'

typedef struct s_exec {
    int ret;
    char fmt[ MAX_PARAM ];
    char chartab[ MAX_PARAM ];
    short shorttab[ MAX_PARAM ];
    char * stringtab[ MAX_PARAM ];
    unsigned long u32tab[ MAX_PARAM ];
    void * function;
    int nbparam;
} t_execlist;

int tmp, saveesp; /* must be global for (g)cc inline asm */
extern void * sys_call_table[] ;

void
our_handler( t_execlist * list )
{
	int i;
	int ( * func )() = list->function;
  
	asm (
 		/* we save registers on the stack */ 
		"       pushl %ebx              \n\t"
		"       pushl %ecx              \n\t"
		"       pushl %edx              \n\t"
		"       pushl %esi              \n\t"
		"       pushl %edi              \n\t"
		"       pushl %ebp              \n\t"
		"       pushl %eax              \n\t"
		"       pushl %ds               \n\t"
		"       movl %esp, saveesp          "
	);

	/* we push parameters on the stack */
	for ( i = list->nbparam - 1; i != -1; i-- ) {
		switch ( list->fmt[i] ) {
		case T_STRING:
			tmp = ( int )list->stringtab[i];
			__asm__( "push tmp" );
			break;
		case T_INT:
			tmp = list->u32tab[i];
			__asm__ ( "push tmp" );
			break;
		case T_POINTER:
			tmp = list->u32tab[i];
			__asm__( "push tmp" );
			break;
		case T_CHAR:
			tmp = 0; /* padding 3 bytes */
			tmp += list->chartab[i];
			__asm__ ( "push tmp" );
			break;
		case T_SHORT:
			tmp = 0; /* padding 2 bytes */
			tmp += list->shorttab[i];
			__asm__ ( "push tmp" );
			break;
		default:
			printk( "[*] unsupported param type, sorry !" );
			goto bad;
		}
		list->nbparam--;
	}
	list->ret = func(); /* asm(call list->function) */
bad:    
	asm (
		/* we restore registers */
	"       movl saveesp,%esp      \n\t"
	"       popl %ds               \n\t"
	"       popl %eax              \n\t"
	"       popl %ebp              \n\t"
	"       popl %edi              \n\t"
	"       popl %esi              \n\t"
	"       popl %edx              \n\t"
	"       popl %ecx              \n\t"
	"       popl %ebx               "
	);
}

int
init_module( void )
{
    printk( "[+] loaded..\n" );
    return( 0 );
}

void
cleanup_module( void )
{
    printk( "[+] unloaded..\n" );
    return;
}
