/* 
 * Created : Sat Dec 21 00:43:53 CET 2002
 * Updated : Thu Jan  2 10:56:54 CET 2003
 *
 * Copyright (C) 2002 sauron@kernsh.org
 * Copyright (C) 2002 zorgon@kernsh.org
 *
 * THIS LICENCE IS ISSUED FROM NMAP LICENCE AND MODIFIED
 * 
 * The program kernsh is (C) 2002 sauron/zorgon. This program is 
 * free software; you can redistribute it and/or modify it under the 
 * terms of the GNU General Public License as published by the Free
 * Software Foundation; Version 2. This guarantees your right to use,
 * modify, and redistribute this software under certain conditions. 
 * 
 * Source is provided to this software because we believe users have a
 * right to know exactly what a program is going to do before they run
 * it.                                                                  
 *
 * Source code also allows you to fix bugs, and add new features.
 * You are highly encouraged to send your changes to dev@kernsh.org for 
 * possible incorporation into the main distribution. By sending these 
 * changes to sauron@kernsh.org or zorgon@kernsh.org, it is assumed that 
 * you are offering them the unlimited, non-exclusive right to reuse, 
 * modify, and relicense the code. This is important because the 
 * inability to relicense code has caused devastating problems for other 
 * Free Software projects (such as KDE and NASM). Kernsh will always be 
 * available Open Source. If you wish to specify special license 
 * conditions of your contributions, just say so when you send them.
 *
 * This program is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License for more details (
 * http://www.gnu.org/copyleft/gpl.html ).
 *
 */

#include "include/kernsh.h"

char *
read_signatures_file( int nr, FILE * fd )
{
	char line[ BUFSIZ ];
	char * p_line;
	char parse[ 5 ];
	int lineno;

	lineno = 0;
	snprintf( parse, sizeof(parse), "%d: ", nr );
	while ( fgets(line, sizeof(line)-1, fd) ) {
		lineno++;

		if ( *line == '\n' || *line == '#' ) {
			continue;
		}

		if ( strstr(line, parse) ) {
			p_line = line + strlen( parse );
			while ( *p_line && isspace((int)*p_line) ) {
				p_line++;
			}

			if ( !*p_line ) {
				fatal( "parse error on line %d : %s\n", lineno, line );
				return( NULL );
			}
			return( p_line );
		}
	}
	return( NULL );
}

void
create_file_signatures( char * banner, char * signatures_file, 
int size_sign, int family )
{
	char buffer[ BUFSIZ ];
	unsigned char md5sum[ BUFSIZ ];
	unsigned char * segment;
	char * md5dump;
	size_t p_interrupt;
	time_t tps;
	int fd_signatures;
	int i;
	int n;

	fd_signatures = open( signatures_file, O_RDWR | O_CREAT | 
	O_EXCL, 0644 );
	if ( !fd_signatures ) {
		perror( "open()" );
		return;
	}

	md5dump = malloc( size_sign+1 );
	if ( md5dump == NULL ) {
		perror( "malloc()" );
		return;
	}

	memset( buffer, '\0', BUFSIZ );
	n = time( &tps );
	if ( n > 0 ) {
		snprintf( buffer, sizeof(buffer), "# %s# %s#\n"
		"# size used : %d\n\n", banner, ctime(&tps), 
		size_sign );
		n = write( fd_signatures, buffer, strlen(buffer) );
		if ( n != strlen(buffer) ) {
			perror( "write()" );
			return;
		}
	}

	/* NR_INT = NR_syscalls = 256 */
	for ( i = 0; i < 256; i++ ) {
		memset( md5dump, '\0', size_sign+1 );
		if ( family == SIG_FOR_IDT ) {
			segment = get_idt_segment( i );
			if ( segment == NULL ) {
				fatal( "archive file not create : can't get "
				"segment\n\n" );
				return;
			}

			p_interrupt = ( idt.off2 << 16 ) + idt.off1;

			/* size of MD5 signatures = size_t */
			n = read_kmem( md5dump, p_interrupt, size_sign, 
			file_descriptor_kmem );
		}

		if ( family == SIG_FOR_SYSCALL ) {
			n = read_kmem( md5dump, sys_call_table[i], size_sign,
			file_descriptor_kmem );
		}

		if ( n < 0 ) {
			fatal( "can't get mem to address %#x (int %d)..\n", 
			i, p_interrupt );
			return;
		}
		
		memset( md5sum, '\0', BUFSIZ );
		dumpmd5( md5dump, size_sign, md5sum );
		snprintf( buffer, sizeof(buffer), "%d: %s\n", i, md5sum );
		n = write( fd_signatures, buffer, strlen(buffer) );
		if ( n != strlen(buffer) ) {
			perror( "write()" );
			close( fd_signatures );
			return;
		}
	}

	fprintf( stdout, "%s created..\n\n", signatures_file );
	free( md5dump );
	close( fd_signatures );
	return;
}

int
check_file_signatures( char * banner, char * signatures_files, 
int size_sign, int family )
{
	char * md5dump;
	char * savmd5sum;
	unsigned char * segment;
	unsigned char md5sum[ BUFSIZ ];
	char c;
	size_t actual_interrupt;
	FILE * fd_signatures;
	int change;
	int i;
	int j;
	int n;

	md5dump = malloc( size_sign+1 );
	if ( md5dump == NULL ) {
		perror( "malloc()" );
		return( -1 );
	}

	change = j = 0;
	fprintf( stdout, "%s", banner );

	/* NR_INT = NR_syscalls = 256 */
	for ( i = 0; i < 256; i++ ) {
		memset( md5dump, '\0', size_sign+1 );
		if ( family == SIG_FOR_IDT ) {
			segment = get_idt_segment( i );
			if ( segment == NULL ) {
				fatal( "can't get segment\n\n" );
				break;
			}

			actual_interrupt = ( idt.off2 << 16 ) + idt.off1;

			n = read_kmem( md5dump, actual_interrupt, size_sign, 
			file_descriptor_kmem );
		}

		if ( family == SIG_FOR_SYSCALL ) {
			n = read_kmem( md5dump, sys_call_table[i], size_sign,
			file_descriptor_kmem );
		}

		if ( n < 0 ) {
			fatal( "can't get mem to address %#x (int %d)..\n", 
			actual_interrupt, i );
			break;
		}

		memset( md5sum, '\0', BUFSIZ );
		dumpmd5( md5dump, size_sign, md5sum );

		fd_signatures = fopen( signatures_files, "r" );
		if ( fd_signatures < 0 ) {
			perror( "open()" );
			return( -1 );
		}

		savmd5sum = read_signatures_file( i, fd_signatures );
		if ( savmd5sum == NULL ) {
			fatal( "no signature for interrupt or syscall %d..\n", i );
			break;
		}

		if ( !strstr(savmd5sum, md5sum) ) {
			fprintf( stdout, "%d\t%s\t\t%s\n", i, savmd5sum, md5sum );
			change++;
			j++;
		}

		if ( !((j+1)%10) ) {
			c = enter();
			if ( c == '\r' || c == '\n' ) {
				continue;
			}
			if ( c == 'q' ) {
				getchar();
				goto end;
			}
		}
	}

	free( md5dump );
	fclose( fd_signatures );

end:
	if ( !change ) {
		return( 0 );
	}
	return( -1 );
}
